@@ -31,7 +31,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.Map;
 import net.sourceforge.guacamole.GuacamoleException;
-import net.sourceforge.guacamole.net.auth.UsernamePassword;
+import net.sourceforge.guacamole.net.auth.Credentials;
 import net.sourceforge.guacamole.properties.FileGuacamoleProperty;
 import net.sourceforge.guacamole.properties.GuacamoleProperties;
 import net.sourceforge.guacamole.protocol.GuacamoleConfiguration;
@@ -46,12 +46,12 @@ import org.xml.sax.helpers.XMLReaderFactory;
 
 /**
  * Authenticates users against a static list of username/password pairs.
- * Each username/password may be associated with exactly one configuration.
+ * Each username/password may be associated with multiple configurations.
  * This list is stored in an XML file which is reread if modified.
  * 
- * @author Michael Jumper
+ * @author Michael Jumper, Michal Kotas
  */
-public class BasicFileAuthenticationProvider implements AuthenticationProvider<UsernamePassword> {
+public class BasicFileAuthenticationProvider implements AuthenticationProvider {
 
     private Logger logger = LoggerFactory.getLogger(BasicFileAuthenticationProvider.class);
     
@@ -113,7 +113,7 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
     }
 
     @Override
-    public Map<String, GuacamoleConfiguration> getAuthorizedConfigurations(UsernamePassword credentials) throws GuacamoleException {
+    public Map<String, GuacamoleConfiguration> getAuthorizedConfigurations(Credentials credentials) throws GuacamoleException {
 
         // Check mapping file mod time
         File userMappingFile = getUserMappingFile();
@@ -135,11 +135,8 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
         
         // Validate and return info for given user and pass
         AuthInfo info = mapping.get(credentials.getUsername());
-        if (info != null && info.validate(credentials.getUsername(), credentials.getPassword())) {
-            Map<String, GuacamoleConfiguration> configs = new HashMap<String, GuacamoleConfiguration>();
-            configs.put("DEFAULT", info.getConfiguration());
-            return configs;
-        }
+        if (info != null && info.validate(credentials.getUsername(), credentials.getPassword()))
+            return info.getConfigurations();
 
         // Unauthorized
         return null;
@@ -157,14 +154,14 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
         private String auth_password;
         private Encoding auth_encoding;
 
-        private GuacamoleConfiguration config;
+        private Map<String, GuacamoleConfiguration> configs;
 
         public AuthInfo(String auth_username, String auth_password, Encoding auth_encoding) {
             this.auth_username = auth_username;
             this.auth_password = auth_password;
             this.auth_encoding = auth_encoding;
 
-            config = new GuacamoleConfiguration();
+            configs = new HashMap<String, GuacamoleConfiguration>();
         }
 
         private static final char HEX_CHARS[] = {
@@ -220,13 +217,25 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
 
         }
 
-        public GuacamoleConfiguration getConfiguration() {
+        public GuacamoleConfiguration getConfiguration(String name) {
+
+            // Create new configuration if not already in map
+            GuacamoleConfiguration config = configs.get(name);
+            if (config == null) {
+                config = new GuacamoleConfiguration();
+                configs.put(name, config);
+            }
+
             return config;
+
+        }
+
+        public Map<String, GuacamoleConfiguration> getConfigurations() {
+            return configs;
         }
 
     }
 
-
     private static class BasicUserMappingContentHandler extends DefaultHandler {
 
         private Map<String, AuthInfo> authMapping = new HashMap<String, AuthInfo>();
@@ -238,15 +247,26 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
         private enum State {
             ROOT,
             USER_MAPPING,
+
+            /* Username/password pair */
             AUTH_INFO,
+
+            /* Connection configuration information */
+            CONNECTION,
             PROTOCOL,
             PARAMETER,
+
+            /* Configuration information associated with default connection */
+            DEFAULT_CONNECTION_PROTOCOL,
+            DEFAULT_CONNECTION_PARAMETER,
+
             END;
         }
 
         private State state = State.ROOT;
         private AuthInfo current = null;
         private String currentParameter = null;
+        private String currentConnection = null;
 
         @Override
         public void endElement(String uri, String localName, String qName) throws SAXException {
@@ -277,9 +297,36 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
                     }
 
                     break;
+                    
+                case CONNECTION:
+
+                    if (localName.equals("connection")) {
+                        state = State.AUTH_INFO;
+                        return;
+                    }
+
+                    break;                
 
                 case PROTOCOL:
 
+                    if (localName.equals("protocol")) {
+                        state = State.CONNECTION;
+                        return;
+                    }
+
+                    break;
+
+                case PARAMETER:
+
+                    if (localName.equals("param")) {
+                        state = State.CONNECTION;
+                        return;
+                    }
+
+                    break;
+
+                case DEFAULT_CONNECTION_PROTOCOL:
+
                     if (localName.equals("protocol")) {
                         state = State.AUTH_INFO;
                         return;
@@ -287,7 +334,7 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
 
                     break;
 
-                case PARAMETER:
+                case DEFAULT_CONNECTION_PARAMETER:
 
                     if (localName.equals("param")) {
                         state = State.AUTH_INFO;
@@ -317,7 +364,6 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
 
                     break;
 
-                // Only <authorize> tags allowed in main document
                 case USER_MAPPING:
 
                     if (localName.equals("authorize")) {
@@ -349,6 +395,45 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
 
                 case AUTH_INFO:
 
+                    if (localName.equals("connection")) {
+
+                        currentConnection = attributes.getValue("name");
+                        if (currentConnection == null)
+                            throw new SAXException("Attribute \"name\" required for connection tag.");
+                        
+                        // Next state
+                        state = State.CONNECTION;
+                        return;
+                    }
+
+                    if (localName.equals("protocol")) {
+
+                        // Associate protocol with default connection
+                        currentConnection = "DEFAULT";
+                        
+                        // Next state
+                        state = State.DEFAULT_CONNECTION_PROTOCOL;
+                        return;
+                    }
+
+                    if (localName.equals("param")) {
+
+                        // Associate parameter with default connection
+                        currentConnection = "DEFAULT";
+                        
+                        currentParameter = attributes.getValue("name");
+                        if (currentParameter == null)
+                            throw new SAXException("Attribute \"name\" required for param tag.");
+
+                        // Next state
+                        state = State.DEFAULT_CONNECTION_PARAMETER;
+                        return;
+                    }
+
+                    break;
+                    
+                case CONNECTION:
+
                     if (localName.equals("protocol")) {
                         // Next state
                         state = State.PROTOCOL;
@@ -366,7 +451,7 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
                         return;
                     }
 
-                    break;
+                    break;                   
 
             }
 
@@ -378,15 +463,20 @@ public class BasicFileAuthenticationProvider implements AuthenticationProvider<U
         public void characters(char[] ch, int start, int length) throws SAXException {
 
             String str = new String(ch, start, length);
+   
             switch (state) {
 
                 case PROTOCOL:
-                    current.getConfiguration()
-                            .setProtocol(str);
+                case DEFAULT_CONNECTION_PROTOCOL:
+
+                    current.getConfiguration(currentConnection)
+                        .setProtocol(str);
                     return;
 
                 case PARAMETER:
-                    current.getConfiguration()
+                case DEFAULT_CONNECTION_PARAMETER:
+
+                    current.getConfiguration(currentConnection)
                             .setParameter(currentParameter, str);
                     return;
                 
